"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const ts = require("typescript");
const getOpenApiVersion_1 = require("./utils/getOpenApiVersion");
const getOpenApiSpec_1 = require("./utils/getOpenApiSpec");
const v2_1 = require("./openApi/v2");
const v3_1 = require("./openApi/v3");
const readHandlebarsTemplates_1 = require("./utils/readHandlebarsTemplates");
const writeClient_1 = require("./utils/writeClient");
var Language;
(function (Language) {
    Language["TYPESCRIPT"] = "typescript";
    Language["JAVASCRIPT"] = "javascript";
})(Language = exports.Language || (exports.Language = {}));
var HttpClient;
(function (HttpClient) {
    HttpClient["FETCH"] = "fetch";
    HttpClient["XHR"] = "xhr";
})(HttpClient = exports.HttpClient || (exports.HttpClient = {}));
/**
 * Generate the OpenAPI client. This method will read the OpenAPI specification and based on the
 * given language it will generate the client, including the typed models, validation schemas,
 * service layer, etc.
 * @param input The relative location of the OpenAPI spec.
 * @param output The relative location of the output directory.
 * @param language: The language that should be generated (Typescript or Javascript).
 * @param httpClient: The selected httpClient (fetch or XHR).
 */
function generate(input, output, language = Language.TYPESCRIPT, httpClient = HttpClient.FETCH) {
    const inputPath = path.resolve(process.cwd(), input);
    const outputPath = path.resolve(process.cwd(), output);
    try {
        // Load the specification, read the OpenAPI version and load the
        // handlebar templates for the given language
        const openApi = getOpenApiSpec_1.getOpenApiSpec(inputPath);
        const openApiVersion = getOpenApiVersion_1.getOpenApiVersion(openApi);
        const templates = readHandlebarsTemplates_1.readHandlebarsTemplates(language);
        switch (language) {
            case Language.JAVASCRIPT:
            case Language.TYPESCRIPT:
                // Generate and write version 2 client
                if (openApiVersion === getOpenApiVersion_1.OpenApiVersion.V2) {
                    const clientV2 = v2_1.parse(openApi);
                    writeClient_1.writeClient(clientV2, language, httpClient, templates, outputPath);
                }
                // Generate and write version 3 client
                if (openApiVersion === getOpenApiVersion_1.OpenApiVersion.V3) {
                    const clientV3 = v3_1.parse(openApi);
                    writeClient_1.writeClient(clientV3, language, httpClient, templates, outputPath);
                }
        }
    }
    catch (e) {
        console.error(e);
        process.exit(1);
    }
}
exports.generate = generate;
function compile(dir) {
    const config = {
        compilerOptions: {
            target: 'esnext',
            module: 'commonjs',
            moduleResolution: 'node',
        },
        include: ['./index.ts'],
    };
    const configFile = ts.parseConfigFileTextToJson('tsconfig.json', JSON.stringify(config));
    const configFileResult = ts.parseJsonConfigFileContent(configFile.config, ts.sys, path.resolve(process.cwd(), dir), undefined, 'tsconfig.json');
    const compilerHost = ts.createCompilerHost(configFileResult.options);
    const compiler = ts.createProgram(configFileResult.fileNames, configFileResult.options, compilerHost);
    compiler.emit();
}
exports.compile = compile;
